import 'package:analyzer/dart/element/element.dart';
import 'package:like_sealed_gen/src/util.dart';

class SealedCollectionContent {
  final String className;
  final List<ClassElement> types;

  SealedCollectionContent(this.className, this.types);

  String content() {
    return """
   class ${className}s {
   ${_methods()}
   }
   """;
  }

  String _methods() {
    final methods = List<String>();
    for (var type in types) {
      if (type.isAbstract) {
        continue;
      }
      for (var constructor in type.constructors) {
        if (constructor.isAbstract) {
          continue;
        }
        var functionName = type.name.firstToLower;
        var className = type.name;
        var returnType = constructor.returnType.toString();

        if (constructor.name.isNotEmpty) {
          functionName = functionName + constructor.name.firstToUpper;
          className = className + "." + constructor.name;
        }

        final textConstructor = constructor.parameters.map(
          (parameter) {
            if (parameter.isNamed) {
              return parameter.name + " : " + parameter.name;
            } else {
              return parameter.name;
            }
          },
        ).join(" , ");
        var parameters = "";
        var hasOptional = false;
        var hasNamed = false;
        for (var parameter in constructor.parameters) {
          if (!hasOptional && parameter.isOptional) {
            hasOptional = true;
            if (parameter.isNamed) {
              hasNamed = true;
              parameters += "{";
            } else {
              parameters += "[";
            }
          }
          parameters += parameter.type.getDisplayString();
          parameters += " ";
          parameters += parameter.name;
          parameters += ", ";
        }
        if (hasOptional) {
          if (hasNamed) {
            parameters += "}";
          } else {
            parameters += "]";
          }
        }
        final function = """
        static $returnType $functionName($parameters){
          return $className($textConstructor);
        } 
        """;

        methods.add(function);
      }
    }
    return methods.join("\n");
  }
}
