import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:build/src/builder/build_step.dart';
import 'package:like_sealed/like_sealed.dart';
import 'package:like_sealed_gen/src/content/sealed_content.dart';
import 'package:source_gen/source_gen.dart';

class SealedGenerator extends GeneratorForAnnotation<LikeSealed> {
  @override
  generateForAnnotatedElement(
    Element element,
    ConstantReader annotation,
    BuildStep buildStep,
  ) {
    if (element is ClassElement) {
      final annotationUri = annotation.objectValue.type.element.source.uri.toString();
      final className = element.name;
      final imports = element.library.imports
          .where((item) => item.uri != null && item.uri != annotationUri)
          .map((item) => item.uri)
          .toList();
      imports.add(element.librarySource.uri.toString());
      final extendedClasses = getSubtypes(element.library, className);

      extendedClasses.add(element);

      return ContentFormat().build(
        imports,
        className,
        extendedClasses,
      );
    }
    return "";
  }

  List<ClassElement> getSubtypes(LibraryElement lib, String type) {
    final types = <ClassElement>[];
    final libraryReader = LibraryReader(lib);

    libraryReader.classes.forEach((classElement) {
      if (isExtended(classElement.supertype, type)) {
        types.add(classElement);
      }
    });

    return types;
  }

  bool isExtended(InterfaceType element, String type) {
    if (element == null) {
      return false;
    } else if (element?.getDisplayString() == type) {
      return true;
    } else {
      return isExtended(element.superclass, type);
    }
  }
}
